@@ -38,6 +38,16 @@ def compiler_instance_factory(compiler, dialect, statement):  # noqa: C901
                 return "'" + value.strftime('%Y-%m-%d') + "'"
             elif isinstance(value, datetime.time):
                 return "'{:%H:%M:%S.%f}'".format(value)
+            elif isinstance(value, datetime.timedelta):
+                return str(value)
+            elif isinstance(value, str):
+                return f"'{value}'"
+            elif isinstance(value, list) and isinstance(type_, sqlalchemy.ARRAY):
+                elements = [
+                    self.render_literal_value(list_val, type_.item_type)
+                    for list_val in value
+                ]
+                return f"({', '.join(elements)})"
             elif value is None:
                 return 'NULL'
             else:
