@@ -1,5 +1,7 @@
 from __future__ import absolute_import
 
+import datetime as dt
+import json
 import re
 from decimal import Decimal
 from os import path
@@ -13,6 +15,7 @@ from werkzeug.datastructures import MultiDict
 
 from webgrid import Column, BoolColumn, YesNoColumn
 from webgrid.extensions import (
+    CustomJsonEncoder,
     RequestArgsLoader,
     RequestFormLoader,
     RequestJsonLoader,
@@ -108,9 +111,29 @@ class TestGrid(object):
             for idx, call in enumerate(m_debug.call_args_list):
                 assert re.match(expected[idx], call[0][0])
 
+    def test_subtotal_computed(self):
+        def TestGrid(subtotals_val):
+            class CTG(Grid):
+                subtotals = subtotals_val
+                Column('Sum Total', Person.numericcol.label('something'), has_subtotal='sum')
+            return CTG
+
+        Person.testing_create(numericcol=5)
+        Person.testing_create(numericcol=10)
+
+        assert TestGrid('none')().grand_totals is None
+        assert TestGrid('none')().page_totals is None
+        assert TestGrid('grand')().page_totals is None
+        assert TestGrid('page')().grand_totals is None
+        assert TestGrid('grand')().grand_totals.something == 15
+        assert TestGrid('page')().page_totals.something == 15
+        assert TestGrid('all')().grand_totals.something == 15
+        assert TestGrid('all')().page_totals.something == 15
+
     @mock.patch('logging.Logger.debug')
     def test_subtotal_sum_by_default(self, m_debug):
         class CTG(Grid):
+            subtotals = 'grand'
             Column('Sum Total', Person.numericcol.label('something'), has_subtotal=True)
         Person.testing_create(numericcol=5)
         Person.testing_create(numericcol=10)
@@ -128,6 +151,7 @@ class TestGrid(object):
 
     def test_subtotal_sum(self):
         class CTG(Grid):
+            subtotals = 'grand'
             Column('Sum Total', Person.numericcol.label('something'), has_subtotal='sum')
         Person.testing_create(numericcol=5)
         Person.testing_create(numericcol=10)
@@ -137,6 +161,7 @@ class TestGrid(object):
 
     def test_subtotal_avg(self):
         class CTG(Grid):
+            subtotals = 'grand'
             Column('Sum Total', Person.numericcol.label('something'), has_subtotal='avg')
         Person.testing_create(numericcol=5)
         Person.testing_create(numericcol=10)
@@ -146,6 +171,7 @@ class TestGrid(object):
 
     def test_subtotal_expr_string(self):
         class CTG(Grid):
+            subtotals = 'grand'
             ratio_expr = Person.numericcol / Person.sortorder
             Column('Numeric', Person.numericcol.label('numeric_col'), has_subtotal=True)
             Column('Ints', Person.floatcol.label('float_col'), has_subtotal=True)
@@ -161,6 +187,7 @@ class TestGrid(object):
         sum_ = sasql.functions.sum
 
         class CTG(Grid):
+            subtotals = 'grand'
             ratio_expr = Person.numericcol / Person.sortorder
             Column('Numeric', Person.numericcol.label('numeric_col'), has_subtotal=True)
             Column('Ints', Person.floatcol.label('float_col'), has_subtotal=True)
@@ -671,6 +698,8 @@ class TestQueryStringArgs(object):
         else:
             assert_in_query(pg, 'LIMIT 1 OFFSET 1')
 
+        assert pg.build_qs_args() == 'onpage=2&perpage=1'
+
     @_inrequest('/foo?perpage=5&onpage=foo')
     def test_qs_onpage_invalid(self):
         pg = PeopleGrid()
@@ -678,6 +707,7 @@ class TestQueryStringArgs(object):
         assert pg.on_page == 1
         assert pg.per_page == 5
         assert pg.user_warnings[0] == '"onpage" grid argument invalid, ignoring'
+        assert pg.build_qs_args() == 'perpage=5'
 
     @_inrequest('/foo?perpage=5&onpage=-1')
     def test_qs_onpage_negative(self):
@@ -712,6 +742,7 @@ class TestQueryStringArgs(object):
         assert pg.on_page == 1
         assert pg.per_page == 50
         assert not pg.user_warnings
+        assert pg.build_qs_args() == ''
 
     @_inrequest('/foo?perpage=-1&onpage=2')
     def test_qs_perpage_negative(self):
@@ -727,6 +758,7 @@ class TestQueryStringArgs(object):
         pg.apply_qs_args()
         assert pg.order_by == [('firstname', False), ('status', True)]
         assert pg.user_warnings[0] == 'can\'t sort on invalid key "foo"'
+        assert pg.build_qs_args() == 'sort1=firstname&sort2=-status'
 
     @_inrequest('/foo?sort1=fullname&sort2=firstname&sort3=-status')
     def test_qs_sorting_not_allowed(self):
@@ -759,6 +791,11 @@ class TestQueryStringArgs(object):
         assert pg.columns[4].filter.value1 == [first_id, second_id]
         assert pg.columns[4].filter.value2 is None
 
+        assert pg.build_qs_args() == (
+            'op(firstname)=eq&op(status)=is&v1(firstname)=fn001'
+            f'&v1(status)={first_id}&v1(status)={second_id}'
+        )
+
     @_inrequest('/foo')
     def test_qs_filtering_default_op(self):
         from webgrid_ta.grids import DefaultOpGrid
@@ -1293,3 +1330,24 @@ class TestWebSessionArgsLoader:
         loader.get_args(pg, MultiDict([]))
 
         assert '_PeopleGrid' in flask.session['dgsessions']
+
+
+class TestCustomJSONEncoder:
+    def dump(self, value):
+        return json.dumps(value, cls=CustomJsonEncoder)
+
+    def test_date(self):
+        assert self.dump(dt.date(2022, 5, 31)) == '"2022-05-31"'
+
+    def test_arrow(self):
+        assert self.dump(arrow.get(2022, 5, 31)) == '"2022-05-31T00:00:00+00:00"'
+
+    def test_decimal(self):
+        assert self.dump(Decimal('5')) == '5.0'
+
+    def test_custom_unhandled(self):
+        class Foo:
+            def __str__(self):
+                return 'I am awesome'
+
+        assert self.dump(Foo()) == '"I am awesome"'
